;;;; Anaphora: The Anaphoric Macro Package from Hell
;;;;
;;;; This been placed in Public Domain by the author, 
;;;; Nikodemus Siivola <nikodemus@random-state.net>

(defpackage :anaphora
  (:use :cl)
  (:export
   #:it
   #:aif
   #:sif
   #:asif
   #:awhen
   #:swhen
   #:sunless
   #:acase
   #:scase
   #:aecase
   #:secase
   #:accase
   #:sccase
   #:atypecase
   #:stypecase
   #:aetypecase
   #:setypecase
   #:actypecase
   #:sctypecase
   #:acond
   #:scond))

(in-package :anaphora)

(defmacro anaphoric (op test &body body)
  `(let ((it ,test))
     (,op it ,@body)))

(defmacro symbolic (op test &body body)
  `(symbol-macrolet ((it ,test))
       (,op it ,@body)))

(defmacro aif (test then &optional else)
  `(anaphoric if ,test ,then ,else))

(defmacro sif (test then &optional else)
  `(symbolic if ,test ,then ,else))

(defmacro asif (test then &optional else)
  `(let ((it ,test))
     (if it
	 ,then
	 (symbol-macrolet ((it ,test))
	     ,else))))

(defmacro awhen (test &body body)
  `(anaphoric when ,test ,@body))

(defmacro swhen (test &body body)
  `(symbolic when ,test ,@body))

(defmacro sunless (test &body body)
  `(symbolic unless ,test ,@body))

(defmacro acase (form &body cases)
  `(anaphoric case ,form ,@cases))

(defmacro scase (form &body cases)
  `(symbolic case ,form ,@cases))

(defmacro aecase (form &body cases)
  `(anaphoric ecase ,form ,@cases))

(defmacro secase (form &body cases)
  `(symbolic ecase ,form ,@cases))
  
(defmacro accase (form &body cases)
  `(anaphoric ccase ,form ,@cases))

(defmacro sccase (form &body cases)
  `(symbolic ccase ,form ,@cases))

(defmacro atypecase (form &body cases)
  `(anaphoric typecase ,form ,@cases))

(defmacro stypecase (form &body cases)
  `(symbolic typecase ,form ,@cases))

(defmacro aetypecase (form &body cases)
  `(anaphoric etypecase ,form ,@cases))

(defmacro setypecase (form &body cases)
  `(symbolic etypecase ,form ,@cases))

(defmacro actypecase (form &body cases)
  `(anaphoric ctypecase ,form ,@cases))

(defmacro sctypecase (form &body cases)
  `(symbolic ctypecase ,form ,@cases))

(defmacro acond (&body clauses)
  (labels ((rec (clauses)
	     (if clauses
		 (destructuring-bind ((test &optional result) . rest)  clauses
		   (if result
		       `(anaphoric if ,test ,result ,(rec rest))
		       `(anaphoric if ,test it ,(rec rest))))
		 nil)))
    (rec clauses)))

(defmacro scond (&body clauses)
  (labels ((rec (clauses)
	     (if clauses
		 (destructuring-bind ((test &optional result) . rest) clauses
		   (if result
		       `(symbolic if ,test ,result ,(rec rest))
		       `(symbolic if ,test it ,(rec rest))))
		 nil)))
    (rec clauses)))
